-- run bind variable peek example
-- J.Nemec 2006
-- http://www.db-nemec.com
-- this script demonstrates a instability caused by bind 
--  variable peeking. 
-- Due to peek the partition level statistics from different 
--  partition are used leading to a dramatic plan change
-- 
-- Statement 1 
--- peek decides to use statistics from the partition 2005 (good)
-- Statement 2 
--- peek decides to use statistics from the partition 2006 (bad)
--
begin
declare 
   i number;
   cursor c is
   select entity_id, max(business_year) business_year 
   from my_fact where entity_id between 1 and 1000 
   group by entity_id order by entity_id;
 begin
  for x in c loop  
   update my_fact a
   set status = 'D'
   where entity_id = x.entity_id and business_year = x.business_year
   and version = (select max(version) from my_fact b 
   				  where a.entity_id = b.entity_id and 
					 a.business_year = b.business_year);
   dbms_output.put_line('updating ' || x.entity_id ||
                        ' ' || x.business_year);			  
  end loop;
 end;
end;
/  
-- now define the loop with cursor returning the date - 2006 first
-- note, that the statement must differ from the previous one 
-- to enforce the statement to be parsed with BVP
begin
declare 
   i number;
   cursor c is
   select entity_id, max(business_year) business_year 
   from my_fact where entity_id between 4 and 1000 
   group by entity_id order by entity_id;
 begin
  for x in c loop  
   update my_fact a
   set status = 'O' --- status was changed
   where entity_id = x.entity_id and business_year = x.business_year
   and version = (select max(version) from my_fact b 
   				  where a.entity_id = b.entity_id and 
					 a.business_year = b.business_year);
   dbms_output.put_line('updating ' || x.entity_id ||
                   	    ' ' || x.business_year);			  
  end loop;
 end;
end;
/ 
----
