-- set up the environment for the bind variable peek example
-- J.Nemec 2006
-- http://www.db-nemec.com
-- 
-- the script performs following activities
-- 1) create the fact table with two partitions for business years 2005 and 2006 
-- 2) populate the 2005 partition with great number or records
-- 3) populate the 2006 partition with small number of records
-- 4) create index for access of the entities
-- 5) gather optimizer statistics
--
-- run without parameter (ignore the DROP error if the table doesn't exist
--
DROP TABLE my_fact;
CREATE TABLE  my_fact
(entity_id number,
status varchar2(1),
version number,
business_year date)
PARTITION BY RANGE (business_year)
    ( PARTITION p2005
        VALUES LESS THAN (TO_DATE('2006-01-01','YYYY-MM-DD')),
  PARTITION p2006
        VALUES LESS THAN (TO_DATE('2007-01-01','YYYY-MM-DD'))  
        ); 
--- populate the fact table
-- data for 2005 year
insert into my_fact  
select
trunc(rownum/1000) entity_id, -- entity key
-- Note that the status here is defined only to illustrate the case 
-- there is no deap business logik behind the definition
decode(mod(rownum,2),0,'A','D') status, 
mod(rownum,1000) version, -- version
TO_DATE('2005-01-01','YYYY-MM-DD') -- year 2005
from dual
connect by level < 1000000; 
-- data for 2006 year
insert into my_fact  
select
4 entity_id, -- entity key
decode(mod(rownum,2),0,'A','D') status, 
mod(rownum,1000) version, -- version
TO_DATE('2006-01-01','YYYY-MM-DD') -- year 2006
from dual
connect by level < 1000;     
----
commit;
---
create index my_fact_ix1 on my_fact(entity_id,version) local;
--
exec dbms_stats.gather_table_stats(ownname=>user, granularity => 'all', tabname=>'my_fact', method_opt=>'for all columns size 254', cascade => true);
--
